<input id="{{id}}" type="text" style="border: none; background-color: transparent;" ng-model="copy" readonly>
<i class="fas fa-files-o"
   style="cursor:copy;"
   aria-hidden="true"
   title="Copy to Clipboard"
   ng-click="onCopy()"
   ng-style="style"
></i>
