{% extends 'allianceauth/base.html' %}
{% load i18n %}
{% block page_title %}{% trans app_title %}{% endblock %}

{% block content %}
    <div class="col-lg-12">
        {% include 'standingsrequests/partials/_menu.html' %}        
        <div class="col-lg-12 container">            
            {% block page_content %}{% endblock %}
        </div>
    </div>
{% endblock %}
